#######################################################
#
# Family Tree generation program, v2.0
# Written by Ferenc Bodon and Simon Ward, March 2000 (simonward.com)
# Copyright (C) 2000 Ferenc Bodon, Simon K Ward
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# For a copy of the GNU General Public License, visit 
# http://www.gnu.org or write to the Free Software Foundation, Inc.,
# 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
#######################################################


package ExcelFormat;
use StringUtils;
use Spreadsheet::ParseExcel;
use DataParsers::FieldValidatorParser;
use DataParsers::ExtendedSimonWardFormat; # for getting pictures. Temporal solution
use FamilyTreeData;
use Person;

sub createFamilyTreeDataFromFile {
  my ($config_) = @_;
  my $file_name = $config_->{file_name} or die "No file_name is given in config";
  my $picture_directory =  $config_->{photo_dir} or die "No photo_dir is given in config";
  
  my $family_tree_data = FamilyTreeData->new();
  my $excel = Spreadsheet::ParseExcel::Workbook->Parse($file_name);
  foreach my $sheet (@{$excel->{Worksheet}}) {
    $sheet->{MaxRow} ||= $sheet->{MinRow};
    foreach my $row ($sheet->{MinRow}+1 .. $sheet->{MaxRow}) {
        @fields = map {StringUtils::trim($_)} @fields;
        $family_tree_data->{people}{ $sheet->{Cells}[$row][0]->{Val} } = Person->new( {id => $sheet->{Cells}[$row][0]->{Val}} )
        	if ( !defined $family_tree_data->{people}{ $sheet->{Cells}[$row][0]->{Val}} );
      	my $temp_person = $family_tree_data->{people}{ $sheet->{Cells}[$row][0]->{Val} };
      	
        $temp_person->set_name(Name->new(
          {first_name => $sheet->{Cells}[$row][3]->{Val},
           mid_name   => $sheet->{Cells}[$row][4]->{Val},
           last_name  => $sheet->{Cells}[$row][5]->{Val} }));
        $temp_person->get_name()->set_title($sheet->{Cells}[$row][1]->{Val})
        	if($sheet->{Cells}[$row][1]);
      	$temp_person->get_name()->set_prefix($sheet->{Cells}[$row][2]->{Val})
        	if($sheet->{Cells}[$row][2]);
      	$temp_person->get_name()->set_suffix($sheet->{Cells}[$row][6]->{Val})
        	if($sheet->{Cells}[$row][6]);
      	$temp_person->get_name()->set_nickname($sheet->{Cells}[$row][7]->{Val})
        	if($sheet->{Cells}[$row][7]);
        	
        if($sheet->{Cells}[$row][8]) {
        	$family_tree_data->{people}{ $sheet->{Cells}[$row][8]->{Val} } = 
        	 Person->new( {id => $sheet->{Cells}[$row][8]->{Val}} )
        	 if ( !defined $family_tree_data->{people}{ $sheet->{Cells}[$row][8]->{Val}} );
        	$temp_person->set_father( $family_tree_data->{people}{ $sheet->{Cells}[$row][8]->{Val} } );
          push @{ $family_tree_data->{children}{$sheet->{Cells}[$row][8]->{Val}}}, $temp_person;
        } 
        if($sheet->{Cells}[$row][9]) {
          $family_tree_data->{people}{ $sheet->{Cells}[$row][9]->{Val} } = 
            Person->new( {id => $sheet->{Cells}[$row][9]->{Val}} )
        	 if ( !defined $family_tree_data->{people}{ $sheet->{Cells}[$row][9]->{Val}} );
          $temp_person->set_mother( $family_tree_data->{people}{ $sheet->{Cells}[$row][9]->{Val} } );
          push @{ $family_tree_data->{children}{$sheet->{Cells}[$row][9]->{Val}}}, $temp_person;
        }
        $temp_person->set_email($sheet->{Cells}[$row][10]->{Val}) 
        	if(FieldValidatorParser::validEmail($sheet->{Cells}[$row][10]->{Val}));
      	$temp_person->set_homepage($sheet->{Cells}[$row][11]->{Val}) 
      		if(FieldValidatorParser::validURL($sheet->{Cells}[$row][11]->{Val}));
      	$temp_person->set_date_of_birth(FieldValidatorParser::getDate($sheet->{Cells}[$row][12]->{Val}))
          if ( $sheet->{Cells}[$row][12] );
        $temp_person->set_date_of_death(FieldValidatorParser::getDate($sheet->{Cells}[$row][13]->{Val}))
          if ( $sheet->{Cells}[$row][13]);
        $temp_person->set_gender($sheet->{Cells}[$row][14]->{Val})
        	if( FieldValidatorParser::validBool($sheet->{Cells}[$row][14]->{Val}));
        $temp_person->set_is_living($sheet->{Cells}[$row][15]->{Val})
        	if( FieldValidatorParser::validBool($sheet->{Cells}[$row][15]->{Val}));
      	my $place = FieldValidatorParser::getPlace($sheet->{Cells}[$row][16]->{Val});
      	$temp_person->set_place_of_birth($place)
          if(defined $place);
      	$place = FieldValidatorParser::getPlace($sheet->{Cells}[$row][17]->{Val});
      	$temp_person->set_place_of_death($place)
          if(defined $place);
      	$place = FieldValidatorParser::getCemetery($sheet->{Cells}[$row][18]->{Val});
      	$temp_person->set_cemetery($place)
        	if(defined $place);           
      	$temp_person->set_schools([split( /,/, $sheet->{Cells}[$row][19]->{Val})])
        	if($sheet->{Cells}[$row][19]);        
      	$temp_person->set_jobs([split( /,/, $sheet->{Cells}[$row][20]->{Val} )])
        	if($sheet->{Cells}[$row][20]);
      	$temp_person->set_work_places([split( /,/, $sheet->{Cells}[$row][21]->{Val} )])
        	if($sheet->{Cells}[$row][21]);
      	$temp_person->set_places_of_living(FieldValidatorParser::getPlacesArray($sheet->{Cells}[$row][22]->{Val}))
        	if($sheet->{Cells}[$row][22]);
      	$temp_person->set_general($sheet->{Cells}[$row][23]->{Val})
        	if($sheet->{Cells}[$row][23]);
    }
  }
  ExtendedSimonWardFormat::setPictureDirectory($picture_directory);
  ExtendedSimonWardFormat::fill_up_pictures($family_tree_data);
  return $family_tree_data;
}
1;
